import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:logging/logging.dart';
import 'package:logging_config/logging_config.dart';
import 'package:package_info/package_info.dart';
import 'package:pub_semver/pub_semver.dart' as semver;
import 'package:sunny_dart/info_x.dart';
import 'package:sunny_lifecycle/app_info.dart';
import 'package:uni_links2/uni_links.dart';
import 'package:http/http.dart' as http;

final _log = Logger("initNative");

Future<AppInfo> onAppInit(
    Level rootLogLevel, FallbackAppInfo fallbackInfo) async {
  await configureLogging(
    LogConfig(
      logLevels: {
        "trippiAuth": Level.WARNING,
        "": Level.INFO,
      },
      handler: LoggingHandler.console(),
    ),
  );

  final package = await PackageInfo.fromPlatform();
  String? appStoreUrl;

  final appID = package.packageName;
  appStoreUrl = infoX.isIOS
      ? (await _calculateAppStoreUrl(appID, fallbackInfo))
      : "https://play.google.com/store/apps/details?id=$appID";

  return AppInfo(
    packageName: package.packageName,
    appName: package.appName,
    appBaseUrl: fallbackInfo.appBaseUrl,
    appVersion: semver.Version.parse(package.version),
    buildNumber: package.buildNumber,
    appStoreUrl: appStoreUrl,
  );
}

Future<String?> _calculateAppStoreUrl(
    String bundleId, FallbackAppInfo fallbackInfo) async {
  try {
    final resp = await http
        .get(Uri.parse("http://itunes.apple.com/lookup?bundleId=$bundleId"));
    final bodyInfo = json.decode(resp.body);
    final results = bodyInfo["results"] as List<dynamic>;
    final appStoreUrl = results.first["trackViewUrl"] as String?;
    _log.info("Fetched appStoreUrl=$appStoreUrl");
    return appStoreUrl;
  } catch (e) {
    _log.warning("Unable to calculate app store URL: $e");
    return "https://apps.apple.com/us/app/${fallbackInfo.appleAppName}/id${fallbackInfo.appleAppID}";
  }
}

Future recordFlutterError(FlutterErrorDetails error) async {
  if (!kReleaseMode) {
    // In development mode simply print to console.
    FlutterError.dumpErrorToConsole(error);
  }
//  return Crashlytics.instance.recordFlutterError(error);
  print(error.exception);
  print(error.stack);
}

Future recordDartError(Object error, StackTrace stackTrace) async {
  // Whenever an error occurs, call the `reportCrash` function. This will send
  // Dart errors to our dev console or Crashlytics depending on the environment.
  if (!kReleaseMode) {
    // In development mode simply print to console.
    print(error);
    print(stackTrace);
  }
//  return Crashlytics.instance.recordError(error, stackTrace);
  print(error);
  print(stackTrace);
}

Future<Uri?> getStartingUrl() async {
  final uri = await getInitialUri();
  return uri;
}

Stream<Uri?> getUriChangeStream() {
  return uriLinkStream.where((event) => event != null);
}
