import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:sunny_lifecycle/sunny_lifecycle.dart';

class ProviderBuildContextResolver with SplitRegistrationResolver {
  const ProviderBuildContextResolver();

  @override
  T resolve<T>(BuildContext? context) {
    if (this is T) {
      return this as T;
    }
    try {
      context ??= AppBootstrap.currentContext();
    } catch (e) {
      throw "No current inner context found attempting to resolve $T";
    }
    return Provider.of(context, listen: false);
  }

  @override
  Widget? registerAll(BuildContext context, List<Inst> inits,
      {Widget? child, Key? key}) {
    final x = inits.toList();
    if (x.isNotEmpty) {
      return MultiProvider(
        key: key,
        providers: [
          for (var i in inits) i.toProvider(),
        ],
        child: child,
      );
    } else {
      return child;
    }
  }
}

mixin SplitRegistrationResolver implements BuildContextResolver {
  Widget? registerAll(BuildContext context, List<Inst> inits,
      {Widget? child, Key? key});

  @override
  Widget register(BuildContext context, resolverOrList,
      {Widget? child, Key? key}) {
    Iterable<Inst> extract(final input) {
      if (input is Iterable) {
        return input.expand((e) => extract(e));
      } else if (input is Inst) {
        return [input];
      } else {
        return [Inst.instance(input)];
      }
    }

    final items = extract(resolverOrList).toList();
    return registerAll(context, items, child: child, key: key)!;
  }
}

extension ResolverInitToProviderExt<T extends Object> on Inst<T> {
  Provider toProvider() {
    Provider<X> _toProvider<X extends Object>(Inst<X> inst) {
      return isFactory
          ? Provider<X>(
              create: inst.factory!,
              dispose: ((_, _inst) => inst.dispose?.call(_inst)),
            )
          : Provider<X>.value(
              value: inst.instance!,
              updateShouldNotify: inst.shouldUpdate,
            );
    }

    return this.typed(_toProvider);
  }
}
