import 'dart:convert';

import 'package:crypto/crypto.dart' as crypto;
import 'package:firebase_auth/firebase_auth.dart' as fb;
import 'package:flutter/material.dart';
import 'package:logging/logging.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/sunny_lifecycle.dart';
import 'package:sunny_sdk_core/api/third_party_creds.dart';

class AppleLoginProvider extends LoginProvider {
  final Future Function(BuildContext context)? onWeb;

  const AppleLoginProvider({this.onWeb});

  static final oAuthProvider = fb.OAuthProvider('apple.com');

  final String name = "apple";
  static final _log = Logger("appleLoginHandler");

  @override
  Widget logo(double size, {Color color = Colors.white}) {
    return Icon(
      AuthIcons.appleLogo,
      color: color,
      size: size,
    );
  }

  @override
  Future signOut() async {
    /// This plugin doesn't cache anything, so nothing to do.
  }

  @override
  bool get isEnabled => !infoX.isAndroid;

  @override
  Future<AuthResponse> doAttemptLogin(BuildContext context, {credentials}) async {
    try {
      try {
        final String nonce = uuid();
        final shaOfNonce = crypto.sha256.convert(utf8.encode(nonce)).toString().replaceAll("-", '');

        if (infoX.isIOS || infoX.isMacOS) {
          final credential = await SignInWithApple.getAppleIDCredential(scopes: [
            AppleIDAuthorizationScopes.email,
            AppleIDAuthorizationScopes.fullName,
          ], nonce: shaOfNonce);

          final oauthCred = oAuthProvider.credential(
            idToken: credential.identityToken,
            accessToken: credential.authorizationCode,
            rawNonce: nonce,
          );

          /// Apple only allows one access with the token, so don't do anything with
          /// the token before sending it to Firebase
          return AuthResponse.success(oauthCred);
        } else {
          // ignore: unused_local_variable
          WebAuthenticationOptions? webOptions;
          try {
            final creds = ThirdPartyCreds.appleSignIn.credentials;
            webOptions = WebAuthenticationOptions(
              clientId: creds.clientId!,
              redirectUri: creds.redirectUrl!.toUri()!,
            );
          } catch (e) {
            print(e);
          }
          // var oAuthProvider = fb.OAuthProvider("apple.com")
          //   ..addScope('email')
          //   ..addScope('name')
          //   ..setCustomParameters({
          //     'client_id': webOptions!.clientId,
          //     'redirect_uri': webOptions.redirectUri.toString(),
          //     'scope': 'email name',
          //     // Request `code`, which is also what `ASAuthorizationAppleIDCredential.authorizationCode` contains.
          //     // So the same handling can be used for Apple and 3rd party platforms
          //     'response_type': 'code id_token',
          //     'response_mode': 'form_post',
          //     'nonce': shaOfNonce,
          //   });

          final cred = await fb.FirebaseAuth.instance.signInWithPopup(oAuthProvider);

          var ocred = cred.credential as fb.OAuthCredential;
          return AuthResponse.success(ocred);
          // final oauthCred = oAuthProvider.credential(
          //   idToken: cred.credential.identityToken,
          //   accessToken: credential.authorizationCode,
          //   rawNonce: nonce,
          // );
        }
      } catch (e, stack) {
        _log.severe("Error from server: $e", e, stack);
        return AuthResponse.error(e);
      }
    } catch (e, stack) {
      _log.severe("Error from server: $e", e, stack);
      return AuthResponse.error("There was an error logging you in");
    }
  }
}
