import 'package:flutter/foundation.dart';
import 'package:info_x/info_x.dart';
import 'package:pub_semver/pub_semver.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:sunny_dart/helpers.dart';
import 'package:sunny_lifecycle/app_info.dart';
import 'package:sunny_lifecycle/core_info.dart';
import 'package:sunny_lifecycle/modules/lifecycle_modules.dart';
import 'package:sunny_lifecycle/user_preferences_service.dart';
import 'package:timezone/timezone.dart';

class TestAppInfoModule implements AppInfoModule {
  @override
  Future<AppInfo> loadAppInfo(String identifier, String appTitle) async {
    return AppInfo(
      appStoreUrl: 'https://fakeurl.com/fakeapp',
      appName: 'Test App',
      appVersion: Version.parse('0.0.1'),
      packageName: 'com.test.app',
      buildNumber: '115432',
      appBaseUrl: null,
    );
  }
}

class TestLocalizationModule implements LocalizationModule {
  @override
  Future<InfoX?> initializeInfoX() async {
    return SunnyInfoX();
  }

  @override
  Future<SunnyLocalization?> initializeLocalization(InfoX? infoX) async {
    var mst = TimeZone(-7, isDst: false, abbreviation: 'MST');
    return SunnyLocalization(
        userTimeZone: mst,
        userLocation: Location('America/Phoenix', [], [], [mst]));
  }
}

class TestUserPrefsModule implements UserPrefsModule {
  @override
  UserPreferencesService initializePreferences(String identifier) {
    return UserPreferencesService(
        identifier, () async => TestSharedPreferences());
  }
}

class TestSharedPreferences implements SharedPreferences {
  final _data = <String, Object?>{};

  @override
  bool containsKey(String key) => _data.containsKey(key);

  @override
  Future<bool> commit() => SynchronousFuture(true);

  @override
  Object? get(String key) {
    return _data[key];
  }

  T? getAs<T>(String key) {
    return _data[key] as T?;
  }

  @override
  bool? getBool(String key) => getAs(key);

  @override
  double? getDouble(String key) => getAs(key);

  @override
  int? getInt(String key) => getAs(key);

  @override
  Set<String> getKeys() => _data.keys.toSet();

  @override
  String? getString(String key) => getAs(key);

  @override
  List<String>? getStringList(String key) => getAs(key);

  @override
  Future<void> reload() => SynchronousFuture(null);

  @override
  Future<bool> remove(String key) async {
    return _data.remove(key) != null;
  }

  Future<bool> update(String key, Object? value) async {
    _data[key] = value;
    return true;
  }

  @override
  Future<bool> setBool(String key, bool value) => update(key, value);

  @override
  Future<bool> setDouble(String key, double value) => update(key, value);

  @override
  Future<bool> setInt(String key, int value) => update(key, value);

  @override
  Future<bool> setString(String key, String value) => update(key, value);

  @override
  Future<bool> setStringList(String key, List<String> value) =>
      update(key, value);

  @override
  Future<bool> clear() async {
    _data.clear();
    return true;
  }
}

class TestDeviceInfoModule implements DeviceInfoModule {
  @override
  Future<DeviceInfo> loadDeviceInfo() async {
    return DeviceInfo(
      geo: null,
      isSimulator: false,
      softwareVersion: "unknown",
      locale: "en-US",
      language: "en",
      deviceType: "unknown",
      deviceModel: "unknown",
      deviceBrand: "unknown",
      ipAddress: "unknown",
      deviceId: uuid(),
      software: "unknown",
    );
  }
}
