import 'dart:async';

import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/push/push_interface.dart';
import 'package:sunny_lifecycle/push_notification_service.dart';

PushPlatform pushPlatform() => PushMessagingNative._();

// class PushMessagingNative with PushPlatformMixin, LoggingMixin, Disposable {
//   PushMessagingNative._()
//       : _fb = Firebase.app().let((self) => FirebaseMessaging.instance);
//
//   final FirebaseMessaging _fb;
//   bool _isRegistered;
//
//   bool get isRegistered => _isRegistered;
//
//   @override
//   Future requestPermissions() async {
//     await 200.ms.pause();
//     final settings = await _fb.requestPermission(
//       sound: true,
//       badge: true,
//       alert: true,
//     );
//     this._isRegistered = true;
//   }
//
//   @override
//   Future cancel() async {
//     _fb.configure(
//       onMessage: ignore,
//       onResume: ignore,
//       onBackgroundMessage: null,
//       onLaunch: ignore,
//     );
//   }
//
//   Future close() async {
//     await closePushStreams();
//     await cancel();
//     await super.disposeAll();
//   }
//
//   @override
//   Future subscribe(BuildContext context,
//       {bool configureBackground = false}) async {
//     _fb.configure(
//       onMessage: (Map<String, dynamic> message) async {
//         final msg = extractMessage(message);
//         log.info("OnMessage received: $message");
//         newMessage(PushSource.foreground, msg);
//       },
//       onBackgroundMessage: null,
//       onResume: (Map<String, dynamic> message) async {
//         final msg = extractMessage(message);
//         log.info("OnResume received: $message");
//         newMessage(PushSource.resume, msg);
//       },
//       onLaunch: (Map<String, dynamic> message) async {
//         final msg = extractMessage(message);
//         log.info("OnLaunch received: $message");
//         newMessage(PushSource.resume, msg);
//       },
//     );
//   }
//
//   /// Used for when we're not actively observing things
//   Future ignore(_) async {}
//
//   @override
//   Future initialize(MessageConverter messageConverter) async {
//     _fb
//
//         /// Registers for all token changes
//         .onTokenRefresh
//         .listen((token) {
//       log.info("FCM token refresh: $token");
//       super.newPushToken(token);
//     }, cancelOnError: false).auto(this);
//
//     super.convertMessage = messageConverter;
//   }
//
//   @override
//   Future<String> internalGetToken() {
//     return _fb.getToken();
//   }
// }

class PushMessagingNative with PushPlatformMixin, LoggingMixin, Disposable {
  PushMessagingNative._()
      : _fb = Firebase.app().let((self) => FirebaseMessaging.instance);

  final FirebaseMessaging? _fb;
  bool? _isRegistered;
  bool? get isRegistered => _isRegistered;

  List<StreamSubscription> _subs = [];

  @override
  Future requestPermissions() async {
    await 200.ms.pause();

    final _settings = await _fb!.requestPermission(
      sound: true,
      badge: true,
      alert: true,
    );

    this._isRegistered = _settings.alert == AppleNotificationSetting.enabled;
  }

  @override
  Future cancel() async {
    await _subs.map((sub) => sub.cancel()).waitAll();
  }

  Future close() async {
    await closePushStreams();
    await cancel();
    await super.disposeAll();
  }

  @override
  Future subscribe(BuildContext context,
      {bool configureBackground = false}) async {
    _subs.add(FirebaseMessaging.onMessage.listen((event) {
      final msg = extractMessage(event.data);
      log.info("OnMessage received: ${event.data}");
      newMessage(PushSource.foreground, msg);
    }));
    _subs.add(FirebaseMessaging.onMessageOpenedApp.listen((event) {
      final msg = extractMessage(event.data);
      log.info("OnLaunch received: ${event.data}");
      newMessage(PushSource.resume, msg);
    }));
  }

  /// Used for when we're not actively observing things
  Future ignore(_) async {}

  @override
  Future initialize(MessageConverter messageConverter) async {
    _fb!

        /// Registers for all token changes
        .onTokenRefresh
        .listen((token) {
      log.info("FCM token refresh: $token");
      super.newPushToken(token);
    }, cancelOnError: false).auto(this);

    super.convertMessage = messageConverter;
  }

  @override
  Future<String?> internalGetToken() {
    return _fb!.getToken();
  }
}
