import 'package:sunny_dart/helpers.dart';

class RouteHistoryItem<T extends Object> {
  final String route;
  final T? currTab;
  final bool isPage;

  RouteHistoryItem(this.route, this.currTab, {required this.isPage});

  @override
  String toString() {
    return '$route (from ${currTab ?? 'no focused child'})';
  }
}

typedef Inspector<T> = void Function(T input);

/// Observes the top-level history events
abstract class HistoryObserver {
  void didPushHistory(RouteHistoryItem item, List<RouteHistoryItem> snapshot);

  void didPopHistory(RouteHistoryItem item, List<RouteHistoryItem> snapshot);
}

class RouteHistory<T extends Object> extends ListDelegateMixin<RouteHistoryItem<T>> with LoggingMixin {
  final items = <RouteHistoryItem<T>>[];

  void push(RouteHistoryItem<T> item) {
    items.insert(0, item);
    log.info('-----------');
    log.info('History after push');
    log.info('-----------');
    items.forEach((item) {
      log.info(' - ${item.route} isPage:${item.isPage}, tab: ${item.currTab}');
    });
    log.info('-----------');
  }

  void pop() {
    var popped = items.removeAt(0);
    log.info('<<<<<<<<<<<');
    log.info('History after pop (${popped.route})');
    log.info('<<<<<<<<<<<');
    items.forEach((item) {
      log.info(' - ${item.route} isPage:${item.isPage}, tab: ${item.currTab}');
    });
    log.info('<<<<<<<<<<<');
  }

  List<RouteHistoryItem<T>> match(String route) {
    var results = <RouteHistoryItem<T>>[];
    var found = false;
    //todo: how far back to look?
    for (var item in items.skip(1).take(3)) {
      found = item.route == route;
      if (!found && item.isPage) {
        return [];
      }
      results.add(item);
      if (found) {
        return results;
      }
    }
    return [];
  }

  @override
  List<RouteHistoryItem<T>> get delegate => items;
}
